@@ -46,6 +46,7 @@ __all__ = ["get_dome","get_ccd"]
 def get_ccd(lampfile, ccdspec_mask=None,
             tracematch=None, background=None,
             correct_traceflexure=False, savefile_traceflexure=None,
+            jscan = None,
                 **kwargs):
     """ Load a SEDmachine ccd image.
 
@@ -73,7 +74,8 @@ def get_ccd(lampfile, ccdspec_mask=None,
             from .flexure import get_ccd_jflexure
             from .sedm    import TRACE_DISPERSION
             # Save the flexure plot
-            j_offset = get_ccd_jflexure(lamp, ntraces=200, tracewidth=1, jscan=[-3,3,10],
+            j_offset = get_ccd_jflexure(lamp, ntraces=200, tracewidth=1,
+                                        jscan=[-2.5, 2.5, 10] if jscan is None else jscan,
                                 savefile=savefile_traceflexure, get_object=False)
 
             new_tracematch = lamp.tracematch.get_shifted_tracematch(0, j_offset)
@@ -466,8 +468,10 @@ class CCD( BaseCCD ):
             self.set_background(self._background.background, force_it=True)
 
 
-    def fetch_background(self, set_it=True, build_if_needed=True, **kwargs):
-        """ """
+    def fetch_background(self, set_it=True, build_if_needed=True, ncore=None, **kwargs):
+        """ 
+        ncore is used only if build_background() is called.
+        """
         from .background import load_background
         from .io import filename_to_background_name
         # ---------------- #
@@ -479,7 +483,7 @@ class CCD( BaseCCD ):
             if not build_if_needed:
                 raise IOError("Since build_if_needed=False, No background available.")
             from .background import build_background
-            build_background(self, **kwargs)
+            build_background(self, ncore=ncore, **kwargs)
             warnings.warn("A background has been built")
 
         self._background = load_background( filename_to_background_name( self.filename ))
@@ -653,13 +657,9 @@ class CCD( BaseCCD ):
         # ------------ #
         # - MultiThreading to speed this up
         if show_progress:
-            from astropy.utils.console import ProgressBar
-            ProgressBar.map(_build_ith_flux_, used_indexes)
-        else:
-            #from multiprocessing import Pool as ThreadPool
-            #pool = ThreadPool(4)
-            #pool.map(_build_ith_flux_, used_indexes)
-            _ = [_build_ith_flux_(i) for i in used_indexes]
+            print("show_progress removed from extract_cube()")
+
+        _ = [_build_ith_flux_(i) for i in used_indexes]
 
         cubeflux = np.asarray([cubeflux_[i] for i in used_indexes])
         cubevar  = np.asarray([cubevar_[i]   for i in used_indexes]) if cubevar_ is not None else None
