@@ -9,7 +9,16 @@ import warnings
 from glob import glob
 from astropy.io.fits import getheader, getval
 from astropy.time import Time
-REDUXPATH   = os.getenv('SEDMREDUXPATH',default="~/redux/")
+
+try:
+    from ztfquery.sedm import SEDMLOCAL_BASESOURCE
+    REDUXPATH = os.path.join(SEDMLOCAL_BASESOURCE, "redux")
+except:
+    REDUXPATH   = os.getenv('SEDMREDUXPATH',default="~/redux/")
+
+if REDUXPATH == "~/redux/":
+    warnings.warn(f"Default REDUXPATH {REDUXPATH} is used.")
+    
 _PACKAGE_ROOT = os.path.abspath(os.path.dirname(__file__))+"/"
 SEDM_REDUCER     = os.getenv('SEDM_USER',default="auto")
 
@@ -224,15 +233,32 @@ def fetch_guider(date, filename, astrom=True, extinction=".fits"):
 
 def filename_to_guider(filename, astrom=True, extinction=".fits", nomd5=True):
     """ """
-    date = filename_to_date(filename)
-    id_  = filename_to_id(filename)
-    guiders =  [l for l in os.listdir( get_datapath(date)) if id_ in l and "guider" in l 
-               and extinction in l and (not nomd5 or not l.endswith(".md5"))]
-    if astrom:
-        return [get_datapath(date)+"/"+l for l in guiders if "astrom" in l and (not nomd5 or not l.endswith(".md5"))]
-    return guiders
-    
+    fileinfo = parse_filename(filename)
+    dirname = os.path.dirname(filename)
+    key = "astrom" if astrom else "guider"
+    return [os.path.join(dirname,l) for l in os.listdir( get_datapath(fileinfo["date"]))
+                if fileinfo["sedmid"] in l and key in l and (l.endswith(('.fits', '.gz')) if nomd5 else l) ] 
 
+    
+def parse_filename(filename):
+    """ """
+    filename = filename.split(".")[0]
+    if filename.startswith("crr"):
+        crr, b, ifudate, hh, mm, ss, *targetname  = filename.split("_")
+    else:
+        _, crr, b, ifudate, hh, mm, ss, *targetname = filename.split("_")
+
+    if len(targetname)==0:
+        targetname = None
+    else:
+        targetname = ("-".join(targetname).replace(" ","")).split(".")[0]
+        
+    date = ifudate.replace("ifu","")
+    mjd = Time(f"{date[:4]}-{date[4:6]}-{date[6:]}"+" "+f"{hh}:{mm}:{ss}", format="iso").mjd
+    return {"date":date,
+            "sedmid":f"{ifudate}_{hh}_{mm}_{ss}",
+           "mjd":mjd, 
+           "name":targetname}
 
 def filename_to_background_name(filename):
     """ predefined structure for background naming """
@@ -313,13 +339,35 @@ def load_nightly_tracematch(YYYYMMDD, withmask=False):
             return load_nightly_tracematch(YYYYMMDD, withmask=False)
 
 # - HexaGrid
-def load_nightly_hexagonalgrid(YYYYMMDD):
+def load_nightly_hexagonalgrid(YYYYMMDD, download_it=True,
+                                   nprocess_dl=1, **kwargs):
     """ Load the Grid id <-> QR<->XY position
     This object must have been created. 
+
+    nprocess_dl: [int]
+        Number of // download. 1 means no // processing
+
+
     """
     from .utils.hexagrid import load_hexprojection
-    return load_hexprojection(get_datapath(YYYYMMDD)+"%s_HexaGrid.pkl"%(YYYYMMDD))
+    hexagrid_path = os.path.join(get_datapath(YYYYMMDD),"%s_HexaGrid.pkl"%(YYYYMMDD) )
+    if os.path.isfile(hexagrid_path):
+        return load_hexprojection( hexagrid_path )
 
+    if not download_it:
+        raise IOError(f"Cannot find an hexagrid for date {YYYYMMDD}")
+
+    warnings.warn("cannot find the hexagrid for date {YYYYMMDD}, using ztquery.sedm to download it.")
+    from ztfquery import sedm
+    squery = sedm.SEDMQuery()
+    squery.download_night_calibrations(YYYYMMDD, which="HexaGrid.pkl",
+                                           nprocess=nprocess_dl, **kwargs)
+    # has it been downloaded ?
+    if os.path.isfile(hexagrid_path):
+        return load_hexprojection( hexagrid_path )
+    
+    raise IOError(f"Cannot find an hexagrid for date {YYYYMMDD}, even after calling squery.download_night_calibrations()")
+        
 # - WaveSolution
 def load_nightly_wavesolution(YYYYMMDD, subprocesses=False):
     """ Load the spectral matcher.
@@ -398,8 +446,13 @@ def load_telluric_line(filter=None):
     from .utils.atmosphere import load_telluric_spectrum
     return load_telluric_spectrum(_PACKAGE_ROOT+"data/KPNO_lines.fits", filter=filter)
 
+def get_bad_standard_exposures():
+    """ Get the list of recorded standard star exposures that cannot be used for calibration. """
+    return open(os.path.join(_PACKAGE_ROOT,"data/bad_standard.txt")).read().splitlines()
 
-
+def get_noncalspec_standards():
+    """ Get the list of STD observed by SEDm that are not calspec standards. """
+    return open(os.path.join(_PACKAGE_ROOT,"data/noncalspec_standard.txt")).read().splitlines()
 
 #########################3
 #
