@@ -28,7 +28,7 @@ from ..sedm import INDEX_CCD_CONTOURS, TRACE_DISPERSION, build_sedmcube, build_c
 def build_tracematcher(date, verbose=True, width=None,
                            save_masks=False,
                            rebuild=False,
-                           notebook=False):
+                           ncore=None):
     
     """ Create Spaxel trace Solution 
     This enable to know which pixel belong to which spaxel
@@ -79,7 +79,7 @@ def build_tracematcher(date, verbose=True, width=None,
         if not rebuild and len(glob(timedir+"%s_TraceMatch_WithMasks.pkl"%date))>0:
             warnings.warn("TraceMatch_WithMasks already exists for %s. rebuild is False, so nothing is happening"%date)
             return
-        load_trace_masks(smap, smap.get_traces_within_polygon(INDEX_CCD_CONTOURS), notebook=notebook)
+        load_trace_masks(smap, smap.get_traces_within_polygon(INDEX_CCD_CONTOURS), ncore=ncore)
         smap.writeto(timedir+"%s_TraceMatch_WithMasks.pkl"%date)
     
 ############################
@@ -108,7 +108,7 @@ def build_hexagonalgrid(date, xybounds=None, theta=None):
 ############################
 def build_flatfield(date, lbda_min=7000, lbda_max=9000,
                     ref="dome", build_ref=True,
-                    kind="median", savefig=True):
+                    kind="median", savefig=True, ncore=None):
     """ """
     from ..sedm import get_sedmcube
     from pyifu.spectroscopy  import get_slice
@@ -129,7 +129,7 @@ def build_flatfield(date, lbda_min=7000, lbda_max=9000,
         # - The CCD
         ccdreffile = io.get_night_files(date, kind="ccd.lamp", target=ref)[0]
         ccdref     = get_ccd(ccdreffile, tracematch = tmatch, background = 0)
-        ccdref.fetch_background(set_it=True, build_if_needed=True)
+        ccdref.fetch_background(set_it=True, build_if_needed=True, ncore=ncore)
         if not ccdref.has_var():
             ccdref.set_default_variance()
         # - HexaGrid
@@ -183,8 +183,8 @@ def build_flatfield(date, lbda_min=7000, lbda_max=9000,
 ############################
 def build_backgrounds(date, smoothing=[0,5], start=2, jump=10, 
                         target=None, lamps=True, only_lamps=False, skip_calib=True,
-                        multiprocess=True,
-                        savefig=True, notebook=False, **kwargs):
+                        multiprocess=True, show_progress=False,
+                        savefig=True, ncore=None):
     """ """
     from ..background import build_background
     timedir  = io.get_datapath(date)
@@ -201,9 +201,10 @@ def build_backgrounds(date, smoothing=[0,5], start=2, jump=10,
     print("%d files to go..."%nfiles)
     for i,file_ in enumerate(fileccds):
         build_background(get_ccd(file_, tracematch=tmap, background=0),
-                        start=start, jump=jump, multiprocess=multiprocess, notebook=notebook,
-                        smoothing=smoothing,
-            savefile = None if not savefig else timedir+"bkgd_%s.pdf"%(file_.split('/')[-1].replace(".fits","")))
+                        start=start, jump=jump, multiprocess=multiprocess, 
+                        smoothing=smoothing,show_progress=show_progress,
+                        savefile = None if not savefig else timedir+"bkgd_%s.pdf"%(file_.split('/')[-1].replace(".fits","")),
+                        ncore=ncore)
         
     
 ############################
@@ -213,7 +214,7 @@ def build_backgrounds(date, smoothing=[0,5], start=2, jump=10,
 ############################
 def build_wavesolution(date, verbose=False, ntest=None, idxrange=None,
                        use_fine_tuned_traces=False,
-                       wavedegree=5, contdegree=3,
+                       wavedegree=5, contdegree=3, show_progress=False,
                        lamps=["Hg","Cd","Xe"], savefig=True, saveindividuals=False,
                        xybounds=None, rebuild=True):
     """ Create the wavelength solution for the given night.
@@ -264,7 +265,6 @@ def build_wavesolution(date, verbose=False, ntest=None, idxrange=None,
     idx = idxall if ntest is None else np.random.choice(idxall,ntest, replace=False) 
 
     # - Do The loop and map it thanks to astropy
-    from astropy.utils.console import ProgressBar
     def fitsolution(idx_):
         if saveindividuals:
             saveplot = timedir+"%s_wavesolution_trace%d.pdf"%(date,idx_)
@@ -275,9 +275,22 @@ def build_wavesolution(date, verbose=False, ntest=None, idxrange=None,
         if saveplot is not None:
             csolution._wsol.show(show_guesses=True, savefile=saveplot)
             mpl.close("all")
-            
-    ProgressBar.map(fitsolution, idx)
 
+    if show_progress:
+        from astropy.utils.console import ProgressBar
+        from ..utils import tools
+        notebook = tools.is_running_from_notebook()
+        bar = ProgressBar( len(index_columnidx), ipython_widget=notebook)
+    else:
+        bar = None
+
+    for j,i_ in enumerate(idx):
+        fitsolution(i_)
+        if bar is not None:
+            bar.update(j)
+    if bar is not None:
+        bar.update(len(idx))
+        
     # - output - #
     outfile = "%s_WaveSolution"%date
     if idxrange is not None:
@@ -308,7 +321,7 @@ def build_wavesolution(date, verbose=False, ntest=None, idxrange=None,
 #                          #
 ############################
 def build_night_cubes(date, target=None, lamps=True, only_lamps=False,
-                          skip_calib=True, **kwargs):
+                          skip_calib=True, ncore=None, **kwargs):
     """ 
     """
     fileccds = []
@@ -319,8 +332,8 @@ def build_night_cubes(date, target=None, lamps=True, only_lamps=False,
         if skip_calib: crrfiles = [f for f in crrfiles if "Calib" not in fits.getval(f,"Name")]            
         fileccds += crrfiles
 
-    print(fileccds)
-    build_cubes(fileccds, date, **kwargs)
+    print("fileccds:", fileccds)
+    build_cubes(fileccds, date, ncore=ncore, **kwargs)
 
 
 # ----------------- #
@@ -339,8 +352,9 @@ def build_cubes(ccdfiles,  date, lbda=None,
                 build_calibrated_cube=False, calibration_ref=None,
                 # Out
                 build_guider=True, solve_wcs=False,
-                fileindex=None,
-                savefig=True, verbose=True, notebook=False):
+                fileindex=None, show_progress=False,
+                savefig=True, verbose=True, 
+                ncore=None):
     """ Build a cube from the an IFU ccd image. This image 
     should be bias corrected and cosmic ray corrected.
 
@@ -442,11 +456,12 @@ def build_cubes(ccdfiles,  date, lbda=None,
                               correct_traceflexure = traceflexure_corrected,
                               savefile_traceflexure=flexuresavefile)
         if traceflexure_corrected:
-            load_trace_masks(ccd_.tracematch, ccd_.tracematch.get_traces_within_polygon(INDEX_CCD_CONTOURS),
-                                 notebook=notebook)
+            load_trace_masks(ccd_.tracematch,
+                                ccd_.tracematch.get_traces_within_polygon(INDEX_CCD_CONTOURS),
+                                ncore=ncore)
             
         if not nobackground:
-            ccd_.fetch_background(set_it=True, build_if_needed=True)
+            ccd_.fetch_background(set_it=True, build_if_needed=True, ncore=ncore)
             ccd_.header["CCDBKGD"] = (True, "is the ccd been background subtracted?")
         else:
             ccd_.header["CCDBKGD"] = (False, "is the ccd been background subtracted?")
@@ -482,12 +497,9 @@ def build_cubes(ccdfiles,  date, lbda=None,
             
             
     # The actual build
-    if len(ccds)>1:
-        from astropy.utils.console import ProgressBar
-        ProgressBar.map(_build_cubes_, ccds)
-    else:
-        _build_cubes_(ccds[0])
+    return [_build_cubes_(ccd_) for ccd_ in ccds]
         
+
 # ---------------- #
 # Flux Calibration #
 # ---------------- #
@@ -507,10 +519,11 @@ def calibrate_cubes(cubefiles, date=None, calibrated_reference=None,
                                 calibration_ref=calibrated_reference)
     # - The build
     if len(cubefiles)==1:
-        _build_cal_cubes_(cubefiles[0])
-    else:
-        from astropy.utils.console import ProgressBar
-        ProgressBar.map(_build_cal_cubes_, cubefiles, multiprocess=multiprocess, step=2)
+        return _build_cal_cubes_(cubefiles[0])
+    if multiprocess:
+        print("multiprocess removed for calibrate_cubes()")
+        
+    return [_build_cal_cubes_(cubefiles_) for cubefiles_ in cubefiles]
         
     
 def save_cubeplot(date, kind="cube.basic"):
