@@ -11,7 +11,7 @@ from datetime import datetime
 from propobject           import BaseObject
 # pyifu
 from pyifu.spectroscopy   import Cube, Spectrum, get_spectrum, load_spectrum
-from .utils.tools         import kwargs_update, is_arraylike
+from .utils import tools
 
 from . import io
 
@@ -170,11 +170,18 @@ def get_sedmspec(ascii_data):
     """ """
     from pyifu import get_spectrum
     from astropy.io import fits
+    import ast
     header = fits.Header()
     for d in ascii_data:
         if not d.startswith("#"): continue
-        if "SNID" in d: continue
-        header.set(*d.replace("# ","").split(": "))
+        if "SNID" in d:
+            continue
+        key, value,*_ = d.replace("# ","").split(": ")
+        try:
+            value = ast.literal_eval(value)
+        except:
+            pass # I know
+        header.set(key, value)
 
     spec_data = np.asarray([d.split() for d in ascii_data if not d.startswith("#")], dtype="float").T
     return get_spectrum(*spec_data, header=header)
@@ -329,7 +336,7 @@ def build_sedmcube(ccd, date, lbda=None, flatfield=None,
         flatfield = io.load_nightly_flat(date)
 
     # - Build the Cube
-    cube = ccd.extract_cube(wavesolution, lbda, hexagrid=hexagrid, show_progress=True,
+    cube = ccd.extract_cube(wavesolution, lbda, hexagrid=hexagrid, show_progress=False,
                             pixel_shift=pixel_shift)
 
     # - passing the header inforation
@@ -491,7 +498,7 @@ def get_sedm_flatcube(domecube, s=0.8, loc=4300):
 # ------------------ #
 #  Main Functions    #
 # ------------------ #
-def load_sedmcube(filename, **kwargs):
+def load_sedmcube(filename, apply_byecr=False, **kwargs):
     """Load a Cube from the given filename
 
     Returns
@@ -499,17 +506,26 @@ def load_sedmcube(filename, **kwargs):
     Cube
     """
     # To be split between load and get
-    return get_sedmcube(filename, **kwargs)
+    return get_sedmcube(filename, apply_byecr=apply_byecr, **kwargs)
 
 
-def get_sedmcube(filename, **kwargs):
+def get_sedmcube(filename, apply_byecr=False, **kwargs):
     """ Load a Cube from the given filename
 
     Returns
     -------
     Cube
     """
-    return SEDMCube(filename, **kwargs)
+    cube = SEDMCube(filename, **kwargs)
+    if apply_byecr:
+        try:
+            import json
+            return cube.get_byecr_cube()
+        except (json.JSONDecodeError, OSError, FileNotFoundError) as err:
+            warnings.warn(f"No Hexagrid or corrupted file for {filename}")
+            return cube
+    return cube
+    
 
 def get_aperturespectrum(filename):
     """ Load and return an Aperture Spectrum """
@@ -771,7 +787,7 @@ class SEDMExtractStar( BaseObject ):
     # MAIN    #
     # ------- #
     def run(self, psfmodel=None, slice_width=1,
-                spaxel_unit=IFU_SCALE_UNIT, update=True, **kwargs):
+                spaxel_unit=IFU_SCALE_UNIT, update=True, verbose=False, **kwargs):
         """
         Returns
         -------
@@ -785,11 +801,11 @@ class SEDMExtractStar( BaseObject ):
             self.set_psfmodel(psfmodel)
 
         if not self.has_centroid():
-            print("No target centroid yet, on the set automatically ('auto') ")
+            warnings.warn("No target centroid yet, on the set automatically ('auto') ")
             self.set_centroid("auto")
 
         if not self.is_centroid_in_mla():
-            print("CENTROID NOT IN MLA, extract_star did not run. Default backup solution built")
+            warnings.warn("CENTROID NOT IN MLA, extract_star did not run. Default backup solution built")
             self.build_backup_output()
             return
 
@@ -799,10 +815,12 @@ class SEDMExtractStar( BaseObject ):
                                                             centroids_err=self.centroiderr,
                                                             spaxel_unit = spaxel_unit,
                                                             final_slice_width = slice_width,
-                                                            psfmodel=self.psfmodel, **kwargs)
+                                                            psfmodel=self.psfmodel,
+                                                            verbose=verbose,
+                                                          **kwargs)
         # Divide out exposure time
         expt = spec.header.get("EXPTIME", 1.0)
-        print("Dividing counts by %s seconds" % expt)
+        warnings.warn(f"Dividing counts by {expt} seconds")
         spec.scale_by(expt)
         spec.header.set("CALSCL", True, "Exposure time divided out")
 
@@ -815,21 +833,21 @@ class SEDMExtractStar( BaseObject ):
             return spec, other_es_output
 
     # ExtractStar output
-    def set_es_products(self, spec, cubemodel, psfmodel, bkgdmodel, psffit, slpsf):
+    def set_es_products(self, spec, cubemodel, psfmodel, bkgdmodel, psffit, slpsf,
+                            backup=False):
         """ """
         for k,v in locals().items():
             if k in self.es_products:
                 self.es_products[k] = v # defined just above
 
-        self._build_es_output_()
+        self._build_es_output_(backup=backup)
         self._es_spec_update_()
 
     def build_backup_output(self):
         """ """
-        backup_spec = get_spectrum( SEDM_LBDA, np.ones(len(SEDM_LBDA))*np.NaN, header=self.cube.header )
-        self.set_es_products(backup_spec, None, None, None, None, None)
-        self._build_es_output_(backup=True)
-        self._es_spec_update_()
+        backup_spec = get_spectrum( SEDM_LBDA, np.ones(len(SEDM_LBDA))*np.NaN,
+                                        header=self.cube.header )
+        self.set_es_products(backup_spec, None, None, None, None, None, backup=True)
 
     def _es_spec_update_(self):
         """ """
@@ -840,7 +858,7 @@ class SEDMExtractStar( BaseObject ):
     def _build_es_output_(self, backup=False):
         """ """
         if backup:
-            self._es_headerkey = dict(POSOK = self.is_centroid_in_mla(),
+            self._es_headerkey = dict(posok = self.is_centroid_in_mla(),
                                     lbdaref     = "nan", fwhm_arcsec = "nan",
                                     psf_ab     = "nan", psf_pa      = "nan",
                                     psf_airmass = "nan", psf_chi2    = "nan")
@@ -1027,10 +1045,10 @@ class SEDMExtractStar( BaseObject ):
 
         # - Input check
         if not self.raw_spectrum.header['IMGTYPE'].lower() in ['standard']:
-            raise TypeError("The target is not a standard star (header keyword IMPTYME != standard)")
+            raise TypeError("The target is not a std star (header keyword IMPTYME != standard)")
 
         if 'AIRMASS' not in self.raw_spectrum.header:
-            raise ValueError("The given standard star target has no AIRMASS parameter in the header. Unusable.")
+            raise ValueError("The given std star target has no AIRMASS parameter in the header.")
 
         if self.raw_spectrum.header['QUALITY'] != 0:
             warnings.warn("Standard spectrum of low quality, "+
@@ -1278,8 +1296,8 @@ class SEDMExtractStar( BaseObject ):
         expt = self.spectrum.header['EXPTIME']
         for i,sl_ in enumerate(self.es_products["psffit"].slices.values()):
             color = colors[i]
-            ax.errorbar(np.mean(sl_["lbdarange"]), sl_["slpsf"].fitvalues["amplitude"]/expt,
-                                    yerr=sl_["slpsf"].fitvalues["amplitude.err"]/expt,
+            ax.errorbar(np.mean(sl_["lbdarange"]), sl_["slpsf"].fitvalues["amplitude"]*sl_["slpsf"].fitvalues["used_amplscale"]/expt,
+                                    yerr=sl_["slpsf"].fitvalues["amplitude.err"]*sl_["slpsf"].fitvalues["used_amplscale"]/expt,
                                     marker="o", ls="None",
                                     ecolor="0.7", mfc=color, mec="0.7", ms=10, zorder=5)
             ax.axvspan(*sl_["lbdarange"], color=color, alpha=0.2, zorder=1)
@@ -1625,7 +1643,6 @@ class SEDMCube( Cube ):
     """ SEDM Cube """
     DERIVED_PROPERTIES = ["sky"]
 
-
     def extract_pointsource(self, display=False, displayprop={},
                                 step1range=[4500,7000], step1bins=6,
                                 centroid="auto", prop_position={},
@@ -1654,6 +1671,7 @@ class SEDMCube( Cube ):
         #from . import astrometry
         from shapely import geometry
 
+        if verbose: print(f"* Starting ExtractStar using the '{psfmodel}' model")
         # input convertion
         self.extractstar = SEDMExtractStar(self)
         self.extractstar.set_lbdastep1(lbdarange=step1range, bins=step1bins)
@@ -1667,17 +1685,56 @@ class SEDMCube( Cube ):
             import matplotlib.pyplot as mpl
             self.extractstar.get_humain_input()
             self.extractstar.update_from_humain_input()
+            
         elif spaxels_to_use is not None: # You fixed which you want
             if len(spaxels_to_use)<4:
                 print("WARNING, you provided less than 4 spaxel to be fitted")
             self.extractstar.set_fitted_spaxels(spaxels_to_use)
 
         if self.extractstar.fitted_spaxels is None: # Automatic selections (with or without spaxels)
-            self.extractstar.get_spaxels_tofit(buffer=spaxelbuffer, update=True, spaxels_to_avoid=spaxels_to_avoid)
+            self.extractstar.get_spaxels_tofit(buffer=spaxelbuffer, update=True,
+                                                spaxels_to_avoid=spaxels_to_avoid)
 
         if verbose: print("* Starting extractstar.run")
         return self.extractstar.run(slice_width=slice_width, psfmodel=psfmodel,
-                                        fwhm_guess=fwhm_guess, **kwargs)
+                                        fwhm_guess=fwhm_guess, verbose=verbose, **kwargs)
+
+    def get_night(self):
+        """ """
+        if "ORIGIN" in self.header:
+            night = self.header["ORIGIN"].split("_")[2].replace("ifu","")
+        elif self.filename is not None:
+            night = self.filename.split("/")[-1].split("_")[3].replace("ifu","")
+        else:
+            raise AttributeError("Cannot find the cube's night ; no 'ORIGIN' in the header, no filename.")
+        
+        return night
+    
+    def get_byecr_cube(self, cut_criteria=5):
+        """ """
+        from . import  byecr
+        # This is the storing night
+        night = self.get_night()
+        
+        # Get the affected spaxels
+        byecrcl = byecr.SEDM_BYECR(night, self)
+        cr_df = byecrcl.get_cr_spaxel_info(cut_criteria=cut_criteria)
+        
+        # and NaN their flux.
+        newdata = self.data.copy()
+        newdata[cr_df["cr_lbda_index"], cr_df["cr_spaxel_index"]] = np.nan
+        # Update the header...
+        newheader = self.header.copy()
+        newheader.set("NCR", len(cr_df), "total number of detected cosmic-rays from byecr")
+        newheader.set("NCRSPX", len(np.unique(cr_df["cr_spaxel_index"])), "total number of cosmic-ray affected spaxels")
+        # ..and the filename
+        newfilename = self.filename.replace("e3d_crr","e3d_byecr")
+
+        newcube = self.get_new(newdata=newdata, newheader=newheader)
+        newcube.set_filename(newfilename)
+        return newcube
+
+    
 
     def get_aperture_spec(self, xref, yref, radius, bkgd_annulus=None,
                               refindex=None, adr=True, **kwargs):
@@ -1694,7 +1751,7 @@ class SEDMCube( Cube ):
             sourcex, sourcey = np.ones( len(self.lbda) )*xref,np.ones( len(self.lbda) )*yref
 
         # - Radius
-        if not is_arraylike(radius):
+        if not tools.is_arraylike(radius):
             radius = np.ones(len(self.lbda))*radius
         elif len(radius)!= len(self.lbda):
             raise TypeError("The radius size must be a constant or have the same lenth as self.lbda")
@@ -1796,15 +1853,26 @@ class SEDMCube( Cube ):
         -------
         Void (loads the self.adr)
         """
-        adr_prop = kwargs_update( dict(pressure=pressure,
+        adr_prop = {**dict(pressure=pressure,
                                        lbdaref=lbdaref,
                                        temperature=self.header["IN_AIR"],
                                        relathumidity=self.header["IN_HUM"],
                                        airmass=self.header.get('AIRMASS', 1.1),
                                        parangle=self.header['TEL_PA']),
-                                **kwargs)
+                    **kwargs}
         return super(SEDMCube, self).load_adr(**adr_prop)
 
+    def save_sky(self, nspaxels=50, usemean=False, estimate_from="rawdata",
+                 lbda_range=[5000, 8000], **kwargs):
+        rawsky = self.get_spectrum(
+            self.get_faintest_spaxels(nspaxels, lbda_range=lbda_range,
+                                      **kwargs),
+            usemean=usemean, data=estimate_from)
+        rawsky.set_header(self.header)
+        calsky = flux_calibrate_sedm(rawsky)
+        outfile = self.filename.replace("%s" % io.PROD_CUBEROOT, "skyspec_cal")
+        calsky.writeto(outfile)
+
     def remove_sky(self, nspaxels=50, usemean=False,
                        estimate_from="rawdata", lbda_range=[5000,8000],
                       **kwargs):
@@ -1846,7 +1914,7 @@ class SEDMCube( Cube ):
                  interactive=False, ccd=None,
                  savefile=None, ax=None, show=True,
                  show_meanspectrum=True, cmap=None,
-                 vmin=None, vmax=None, notebook=None,
+                 vmin=None, vmax=None, 
                  **kwargs):
         """ Display the cube.
 
@@ -1902,11 +1970,6 @@ class SEDMCube( Cube ):
         show: [bool] -optional-
             If you do not save the data (see savefile), shall the plot be shown?
 
-        notebook: [bool or None] -optional-
-            Is this running from a notebook?
-            If True, the plot will be made using fig.show() if not with mpl.show()
-            If None, this will try to guess.
-
 
         **kwargs goes to matplotlib's imshow
 
@@ -1914,6 +1977,8 @@ class SEDMCube( Cube ):
         -------
         Void
         """
+        notebook = tools.is_running_from_notebook()
+
         if not interactive or ccd is None:
             return super(SEDMCube, self).show(toshow=toshow, interactive=interactive,
                                            savefile=savefile, ax=ax, show=show,
@@ -2017,7 +2082,7 @@ class ApertureSpectrum( Spectrum ):
         -------
         Void, affect the object (data, variance)
         """
-        if not is_arraylike(coef) or len(coef)==1 or np.shape(coef)==self.data.shape:
+        if not tools.is_arraylike(coef) or len(coef)==1 or np.shape(coef)==self.data.shape:
 
             self._properties["rawdata"]  = self.rawdata / coef
             if self.has_variance():
