@@ -11,7 +11,7 @@ from astropy.utils.console import ProgressBar
 
 from propobject import BaseObject
 from .ccd import get_dome, ScienceCCD
-from .utils.tools import kwargs_update, is_arraylike
+from .utils import tools
 
 from .sedm import SEDM_CCD_SIZE
 try:
@@ -101,7 +101,7 @@ def polygon_mask(vertices, width=2048, height=2048,
     back = Image.new('RGBA', (width, height), BACKCOLOR)
     mask = Image.new('RGBA', (width, height))
     # PIL *needs* (!!) [(),()] format [[],[]] wont work
-    if not is_arraylike(vertices[0][0]):
+    if not tools.is_arraylike(vertices[0][0]):
         vertices = [vertices]
         
     if facecolor is None:
@@ -186,7 +186,7 @@ def get_boxing_polygone(x, y, rangex, width,
     """ """
     from shapely import geometry
     import modefit
-    if not is_arraylike(dy):
+    if not tools.is_arraylike(dy):
         dy = np.ones(len(y))*dy
     pfit = modefit.get_polyfit(x, y, dy, degree=polydegree)
     pfit.fit(a0_guess=np.median(y))
@@ -240,22 +240,30 @@ def verts_to_mask(verts):
     
 
 def load_trace_masks(tmatch, trace_indexes=None, multiprocess=True,
-                         notebook=True, ncore=None):
+                         ncore=None, show_progress=False):
     """ """
     if trace_indexes is None:
         trace_indexes = tmatch.trace_indexes
     
     if multiprocess:
         import multiprocessing
-        bar = ProgressBar( len(trace_indexes), ipython_widget=notebook)
+        if show_progress:
+            notebook = tools.is_running_from_notebook()
+            bar = ProgressBar( len(trace_indexes), ipython_widget=notebook)
+        else:
+            bar = None
+            
         if ncore is None:
             ncore = np.max([multiprocessing.cpu_count() - 1, 1])
-                
+        
         p = multiprocessing.Pool(ncore)
-        for j, mask in enumerate( p.imap(verts_to_mask, [tmatch.trace_vertices[i_] for i_ in trace_indexes])):
+        for j, mask in enumerate( p.imap(verts_to_mask, [tmatch.trace_vertices[i_]
+                                                             for i_ in trace_indexes])):
             tmatch.set_trace_masks(sparse.csr_matrix(mask), trace_indexes[j])
-            bar.update(j)
-        bar.update( len(trace_indexes) )
+            if bar is not None:
+                bar.update(j)
+        if bar is not None:
+            bar.update( len(trace_indexes) )
         
     else:
         raise NotImplementedError("Use multiprocess = True (load_trace_masks)")
@@ -388,7 +396,7 @@ class TraceMatch( BaseObject ):
     # => Deriv the rest
     def set_trace_masks(self, masks, trace_indexes):
         """ Attach to the current instance masks. """
-        if is_arraylike(trace_indexes):
+        if tools.is_arraylike(trace_indexes):
             if len(masks) != len(trace_indexes):
                 raise ValueError("masks and trace_indexes do not have the same size.")
             for i,v in zip(trace_indexes, masks):
@@ -520,7 +528,7 @@ class TraceMatch( BaseObject ):
         **kwargs goes to spectralmatching.get_boxing_polygone() 
         """
         xbounds = self.get_trace_xbounds(traceindex)
-        prop = kwargs_update( dict(dy=1), **kwargs )
+        prop = { **dict(dy=1), **kwargs }
         return np.asarray(get_boxing_polygone(x, y, rangex=np.linspace(xbounds[0], xbounds[1], polydegree+5),
                                     width= width, polydegree=polydegree, get_vertices=True, **prop))
     
@@ -548,7 +556,7 @@ class TraceMatch( BaseObject ):
         x, y 
         """
         xbounds = self.get_trace_xbounds(traceindex)
-        prop = kwargs_update( dict(dy=1), **kwargs )
+        prop = {**dict(dy=1), **kwargs}
         return np.asarray(get_boxing_polygone(x, y, rangex=np.arange(*xbounds),
                                     width= 0, polydegree=polydegree, get_vertices=True, **prop))
 
@@ -685,10 +693,10 @@ class TraceMatch( BaseObject ):
         """ """
         from matplotlib import patches
         # Several indexes given
-        if is_arraylike(traceindex):
-            if not is_arraylike(facecolors):
+        if tools.is_arraylike(traceindex):
+            if not tools.is_arraylike(facecolors):
                 facecolors = [facecolors]*len(traceindex)
-            if not is_arraylike(edgecolors):
+            if not tools.is_arraylike(edgecolors):
                 edgecolors = [edgecolors]*len(traceindex)
                 
             ps = [patches.Polygon(self.trace_vertices[idx_],
