@@ -246,8 +246,11 @@ def fit_spaxel_wavelesolution(arccollection, sequential=True,
     arccollection.fit_wavelengthsolution(wavedegree, legendre=False)
     return arccollection
 
-def fit_wavesolution(lamps, indexes, multiprocess=True, notebook=False):
+def fit_wavesolution(lamps, indexes, multiprocess=True):
     """ """
+    from .utils import tools
+    notebook = tools.is_running_from_notebook()
+
     if notebook:
         print("RUNNING FROM NOTEBOOK")
         try:
